/* 
 * noise_mod.c
 *
 * wave module
 *
 * Copyright (C) 1998 Hermann Samso
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ncurses.h>
#include "types.h"
#include "delfin.h"
#include "display.h"
#include "wave.h"
#include "module.h"
#include "audiodrv.h"

SAMPLE noises[MAXWAVES];
BOOL noise_lock = FALSE;	// change both channels?
BOOL noise_lock_lock = FALSE;

void NoiseAddLeft(FLOAT left);
void NoiseAddRight(FLOAT right);
void NoiseToWave(WAVE* wave, FLOAT left, FLOAT right);

void NoiseInit(void) {
	if (trace_on)
		fprintf(stderr, "NoiseInit()\n");


	ModuleDisplay();
}


void NoiseExit(void) {
	if (trace_on)
		fprintf(stderr, "NoiseExit()\n");
}


void NoiseCtrl(INT input) {
	CHAR str[43];
	FLOAT value;
	INT keys[] = {
		'l',		// 'noise_lock' toggle
		's',		// set l&r noise value
		'1', 		// augment noise value left channel
		'2',
		'3',
		'4',
		'5',
		'6',		// diminish noise value left channel
		'7',
		'8',
		'9',
		'0',
		'q',		// augment noise value right channel
		'w',
		'e',
		'r',
		't',
		'z',		// diminish noise value right channel
		'u',
		'i',
		'o',
		'p'
	};


	if (trace_on)
		fprintf(stderr, "NoiseCtrl()\n");
		
	if (input == keys[0]) {
		if (noise_lock)
			noise_lock = FALSE;
		else
			noise_lock = TRUE;
	}
	if (input == keys[1]) {
		echo();
		notimeout(stdscr, TRUE);
		mvprintw(LINES-1, 0, "noise value:");
		mvgetnstr(LINES-1, 5, str, 42);
		notimeout(stdscr, FALSE);
		noecho();
		value = atof(str);
		noises[WaveGetIndex()].left = value;
		noises[WaveGetIndex()].right = value;
	}
	else if (input == keys[2])
		NoiseAddLeft(0.5);
	else if (input == keys[3])
		NoiseAddLeft(0.25);
	else if (input == keys[4])
		NoiseAddLeft(0.1);
	else if (input == keys[5])
		NoiseAddLeft(0.05);
	else if (input == keys[6])
		NoiseAddLeft(0.01);
	else if (input == keys[7])
		NoiseAddLeft(-0.01);
	else if (input == keys[8])
		NoiseAddLeft(-0.05);
	else if (input == keys[9])
		NoiseAddLeft(-0.1);
	else if (input == keys[10])
		NoiseAddLeft(-0.25);
	else if (input == keys[11])
		NoiseAddLeft(-0.5);
	else if (input == keys[12])
		NoiseAddRight(0.5);
	else if (input == keys[13])
		NoiseAddRight(0.25);
	else if (input == keys[14])
		NoiseAddRight(0.1);
	else if (input == keys[15])
		NoiseAddRight(0.05);
	else if (input == keys[16])
		NoiseAddRight(0.01);
	else if (input == keys[17])
		NoiseAddRight(-0.01);
	else if (input == keys[18])
		NoiseAddRight(-0.05);
	else if (input == keys[19])
		NoiseAddRight(-0.1);
	else if (input == keys[20])
		NoiseAddRight(-0.25);
	else if (input == keys[21])
		NoiseAddRight(-0.5);
	else
		usleep(200);
		
	ModuleDisplay();
}

void NoiseExec(void) {
	INT i;
	INT nr_waves = WaveHowMany();
	WAVE* wave;
	
	if (trace_on)
		fprintf(stderr, "NoiseExec()\n");

	for (i = 0; i < nr_waves; i++) {
		wave = WaveGetByIndex(i);
		if (wave->active)
			NoiseToWave(wave, noises[i].left, noises[i].right);
	}
}



void NoiseDisplay(void) {
	INT i = WaveGetIndex();

	if (trace_on)
		fprintf(stderr, "NoiseDisplay()\n");

	DisplayClearBottom();
	mvprintw(LINES-2, 0, "[%s]", noise_mod.name);
	mvprintw(LINES-2, 8, "l:%f", noises[i].left);
	mvprintw(LINES-1, 0, "lock:%d", noise_lock);
	mvprintw(LINES-1, 8, "r:%f", noises[i].right);
}


void NoiseToWave(WAVE* wave, FLOAT left, FLOAT right) {
	INT i;
	FLOAT value;
		
	if (trace_on)
		fprintf(stderr, "NoiseToWave(%s, %f, %f)\n", wave->name, left, right);

	if (wave == NULL)
		return;
	if (left == 0.0 && right == 0.0)
		return;

	for (i = 0; i < AudioGetBufferSize(); i++) {
		value = left * (rand()/(RAND_MAX+1.0) - 0.5) + 1;
		wave->buffer[i].left = wave->buffer[i].left * value;
		value = right * (rand()/(RAND_MAX+1.0) - 0.5) + 1;
		wave->buffer[i].right = wave->buffer[i].right * value;
	}
	
}

		
void NoiseAddLeft(FLOAT left) {
	INT i = WaveGetIndex();

	if (trace_on)
		fprintf(stderr, "NoiseAddLeft(%f)\n", left);

	noises[i].left = noises[i].left+left;
	if (noise_lock && !noise_lock_lock) {
		noise_lock_lock = TRUE;
		NoiseAddRight(left);
	}
	noise_lock_lock = FALSE;
}


void NoiseAddRight(FLOAT right) {
	INT i = WaveGetIndex();
	
	if (trace_on)
		fprintf(stderr, "NoiseAddRight(%f)\n", right);

	
	noises[i].right = noises[i].right+right;
	if (noise_lock && !noise_lock_lock) {
		noise_lock_lock = TRUE;
		NoiseAddLeft(right);
	}
	noise_lock_lock = FALSE;
}


MODULE noise_mod = {
	NULL,			/* next MODULE  in list */
	NULL,			/* previous MODULE in list */
	"noise",
	"v1998.2.10 by Hermann Samso",
	NoiseInit,
	NoiseExit,
	NoiseCtrl,
	NoiseExec,
	NoiseDisplay
};

